﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Knapsack {
  /// <summary>
  /// An operator that performs similarity calculation between two knapsack solutions.
  /// </summary>
  [Item("KnapsackSimilarityCalculator", "An operator that performs similarity calculation between two knapsack solutions.")]
  [StorableClass]
  public sealed class KnapsackSimilarityCalculator : SimilarityCalculator {
    [StorableConstructor]
    private KnapsackSimilarityCalculator(bool deserializing) : base(deserializing) { }
    private KnapsackSimilarityCalculator(KnapsackSimilarityCalculator original, Cloner cloner) : base(original, cloner) { }
    public KnapsackSimilarityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new KnapsackSimilarityCalculator(this, cloner);
    }

    protected override double CalculateSimilarity(IScope left, IScope right) {
      BinaryVector sol1 = left.Variables[TargetParameter.ActualName].Value as BinaryVector;
      BinaryVector sol2 = right.Variables[TargetParameter.ActualName].Value as BinaryVector;

      double similarity = 0.0;
      for (int i = 0; i < sol1.Length; i++)
        if (sol1[i] == sol2[i]) similarity++;
      return similarity / sol1.Length;
    }
  }
}
